@@ -1,18 +1,20 @@
-from __future__ import absolute_import, division, print_function
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+
 import os
 import warnings
 
-import six
-import py
 import attr
+import py
+import six
 
-import _pytest
 import _pytest._code
 from _pytest.compat import getfslineno
+from _pytest.mark.structures import MarkInfo
+from _pytest.mark.structures import NodeKeywords
 from _pytest.outcomes import fail
 
-from _pytest.mark.structures import NodeKeywords, MarkInfo
-
 SEP = "/"
 
 tracebackcutdir = py.path.local(_pytest.__file__).dirpath()
@@ -25,7 +27,7 @@ def _splitnode(nodeid):
         ''
         'testing/code'
         'testing/code/test_excinfo.py'
-        'testing/code/test_excinfo.py::TestFormattedExcinfo::()'
+        'testing/code/test_excinfo.py::TestFormattedExcinfo'
 
     Return values are lists e.g.
         []
@@ -37,7 +39,7 @@ def _splitnode(nodeid):
         # If there is no root node at all, return an empty list so the caller's logic can remain sane
         return []
     parts = nodeid.split(SEP)
-    # Replace single last element 'test_foo.py::Bar::()' with multiple elements 'test_foo.py', 'Bar', '()'
+    # Replace single last element 'test_foo.py::Bar' with multiple elements 'test_foo.py', 'Bar'
     parts[-1:] = parts[-1].split("::")
     return parts
 
@@ -45,7 +47,7 @@ def _splitnode(nodeid):
 def ischildnode(baseid, nodeid):
     """Return True if the nodeid is a child node of the baseid.
 
-    E.g. 'foo/bar::Baz::()' is a child of 'foo', 'foo/bar' and 'foo/bar::Baz', but not of 'foo/blorp'
+    E.g. 'foo/bar::Baz' is a child of 'foo', 'foo/bar' and 'foo/bar::Baz', but not of 'foo/blorp'
     """
     base_parts = _splitnode(baseid)
     node_parts = _splitnode(nodeid)
@@ -105,10 +107,12 @@ class Node(object):
         self._name2pseudofixturedef = {}
 
         if nodeid is not None:
+            assert "::()" not in nodeid
             self._nodeid = nodeid
         else:
-            assert parent is not None
-            self._nodeid = self.parent.nodeid + "::" + self.name
+            self._nodeid = self.parent.nodeid
+            if self.name != "()":
+                self._nodeid += "::" + self.name
 
     @property
     def ihook(self):
@@ -445,7 +449,7 @@ class Collector(Node):
 def _check_initialpaths_for_relpath(session, fspath):
     for initial_path in session._initialpaths:
         if fspath.common(initial_path) == initial_path:
-            return fspath.relto(initial_path.dirname)
+            return fspath.relto(initial_path)
 
 
 class FSCollector(Collector):
@@ -521,13 +525,7 @@ class Item(Node):
             return self._location
         except AttributeError:
             location = self.reportinfo()
-            # bestrelpath is a quite slow function
-            cache = self.config.__dict__.setdefault("_bestrelpathcache", {})
-            try:
-                fspath = cache[location[0]]
-            except KeyError:
-                fspath = self.session.fspath.bestrelpath(location[0])
-                cache[location[0]] = fspath
+            fspath = self.session._node_location_to_relpath(location[0])
             location = (fspath, location[1], str(location[2]))
             self._location = location
             return location
