@@ -1,13 +1,16 @@
-from __future__ import absolute_import, division, print_function
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
 
 import sys
 import warnings
 from contextlib import contextmanager
 
 import pytest
-
 from _pytest import compat
 
+SHOW_PYTEST_WARNINGS_ARG = "-Walways::pytest.RemovedInPytest4Warning"
+
 
 def _setoption(wmod, arg):
     """
@@ -67,27 +70,29 @@ def catch_warnings_for_item(config, ihook, when, item):
 
     Each warning captured triggers the ``pytest_warning_captured`` hook.
     """
-    args = config.getoption("pythonwarnings") or []
+    cmdline_filters = config.getoption("pythonwarnings") or []
     inifilters = config.getini("filterwarnings")
     with warnings.catch_warnings(record=True) as log:
-        filters_configured = args or inifilters or sys.warnoptions
 
-        for arg in args:
-            warnings._setoption(arg)
+        if not sys.warnoptions:
+            # if user is not explicitly configuring warning filters, show deprecation warnings by default (#2908)
+            warnings.filterwarnings("always", category=DeprecationWarning)
+            warnings.filterwarnings("always", category=PendingDeprecationWarning)
 
+        warnings.filterwarnings("error", category=pytest.RemovedInPytest4Warning)
+
+        # filters should have this precedence: mark, cmdline options, ini
+        # filters should be applied in the inverse order of precedence
         for arg in inifilters:
             _setoption(warnings, arg)
 
+        for arg in cmdline_filters:
+            warnings._setoption(arg)
+
         if item is not None:
             for mark in item.iter_markers(name="filterwarnings"):
                 for arg in mark.args:
                     _setoption(warnings, arg)
-                    filters_configured = True
-
-        if not filters_configured:
-            # if user is not explicitly configuring warning filters, show deprecation warnings by default (#2908)
-            warnings.filterwarnings("always", category=DeprecationWarning)
-            warnings.filterwarnings("always", category=PendingDeprecationWarning)
 
         yield
 
@@ -123,7 +128,7 @@ def warning_record_to_str(warning_message):
     if unicode_warning:
         warnings.warn(
             "Warning is using unicode non convertible to ascii, "
-            "converting to a safe representation:\n  %s" % msg,
+            "converting to a safe representation:\n  {!r}".format(compat.safe_str(msg)),
             UnicodeWarning,
         )
     return msg
@@ -155,7 +160,7 @@ def pytest_terminal_summary(terminalreporter):
         yield
 
 
-def _issue_config_warning(warning, config):
+def _issue_config_warning(warning, config, stacklevel):
     """
     This function should be used instead of calling ``warnings.warn`` directly when we are in the "configure" stage:
     at this point the actual options might not have been set, so we manually trigger the pytest_warning_captured
@@ -163,10 +168,11 @@ def _issue_config_warning(warning, config):
 
     :param warning: the warning instance.
     :param config:
+    :param stacklevel: stacklevel forwarded to warnings.warn
     """
     with warnings.catch_warnings(record=True) as records:
         warnings.simplefilter("always", type(warning))
-        warnings.warn(warning, stacklevel=2)
+        warnings.warn(warning, stacklevel=stacklevel)
     config.hook.pytest_warning_captured.call_historic(
         kwargs=dict(warning_message=records[0], when="config", item=None)
     )
